from django.contrib import admin

from .models import InactivityPing, InactivityPingConfig, LeaveOfAbsence, Webhook

# Register your models here.


@admin.register(InactivityPingConfig)
class InactivityPingConfigAdmin(admin.ModelAdmin):
    list_display = ("name", "days", "_groups", "_states")
    filter_horizontal = ("groups", "states")

    def _groups(self, obj):
        return ", ".join(obj.groups.values_list("name", flat=True))

    def _states(self, obj):
        return (", ".join(obj.states.values_list("name", flat=True)),)


@admin.register(InactivityPing)
class InactivityPingAdmin(admin.ModelAdmin):
    list_display = ("config", "user", "timestamp")

    def has_add_permission(self, request):
        return False

    def has_change_permission(self, request, obj=None):
        return False


def approve_leaveofabsence(modeladmin, request, queryset):
    queryset.update(approver=request.user)


approve_leaveofabsence.short_description = "Approve selected leave of absences"


@admin.register(LeaveOfAbsence)
class LeaveOfAbsenceAdmin(admin.ModelAdmin):
    list_display = ("user", "start", "end", "approver")
    ordering = ("-start",)
    actions = [approve_leaveofabsence]


@admin.register(Webhook)
class WebhookAdmin(admin.ModelAdmin):
    list_display = ("name", "webhook_type", "url")
    filter_horizontal = ("ping_configs",)



from django.utils.translation import gettext_lazy as _

from allianceauth import hooks
from allianceauth.services.hooks import MenuItemHook, UrlHook

from . import urls


class LeaveOfAbsenceMenuItem(MenuItemHook):
    def __init__(self):
        MenuItemHook.__init__(
            self,
            _("Leave of Absence"),
            "fas fa-business-time fa-fw",
            "inactivity:index",
            navactive=["inactivity:"],
        )

    def render(self, request):
        if request.user.has_perm("inactivity.basic_access"):
            return MenuItemHook.render(self, request)
        return ""


@hooks.register("menu_item_hook")
def register_menu():
    return LeaveOfAbsenceMenuItem()


@hooks.register("url_hook")
def register_urls():
    return UrlHook(urls, "inactivity", r"^inactivity/")



# Generated by Django 3.1.2 on 2021-01-18 01:50

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("inactivity", "0001_initial"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="general",
            options={
                "default_permissions": (),
                "managed": False,
                "permissions": (
                    ("basic_access", "Can access this app"),
                    ("manage_leave", "Can manage leave of absence requests"),
                ),
            },
        ),
    ]



import dhooks_lite
from multiselectfield import MultiSelectField

from django.contrib.auth.models import Group, User
from django.core.exceptions import ValidationError
from django.db import models
from django.db.models import Q
from django.utils.translation import gettext_lazy as _

from allianceauth.authentication.models import State


class General(models.Model):
    """Meta model for app permissions"""

    class Meta:
        managed = False
        default_permissions = ()
        permissions = (
            ("basic_access", "Can access this app"),
            ("manage_leave", "Can manage leave of absence requests"),
        )


class InactivityPingConfig(models.Model):
    name = models.CharField(
        max_length=48,
        unique=True,
        help_text=_("Internal name for the inactivity policy. Must be unique."),
    )
    days = models.PositiveIntegerField(
        help_text=_("The number of days the user must be inactive.")
    )
    text = models.TextField(
        help_text=_("The text of the message or notification sent to the end user.")
    )

    groups = models.ManyToManyField(
        Group,
        blank=True,
        help_text=_(
            "Groups subject to the inactivity policy. If empty, applies to all groups."
        ),
        related_name="+",
    )

    states = models.ManyToManyField(
        State,
        blank=True,
        help_text=_(
            "States subject to the inactivity policy. If empty, applies to all states."
        ),
        related_name="+",
    )

    class Meta:
        default_permissions = ()
        verbose_name = "inactivity policy"
        verbose_name_plural = "inactivity policies"

    def __str__(self):
        return _("inactivity policy: %(name)s") % {"name": self.name}

    def is_applicable_to(self, user: User) -> bool:
        is_applicable = True
        if self.groups.count() > 0:
            is_applicable &= self.groups.filter(user=user).count() > 0
        if self.states.count() > 0:
            is_applicable &= self.states.filter(userprofile=user.profile).count() > 0
        return is_applicable


class InactivityPing(models.Model):
    config = models.ForeignKey(InactivityPingConfig, on_delete=models.CASCADE)
    timestamp = models.DateTimeField()
    user = models.ForeignKey(User, on_delete=models.CASCADE)

    class Meta:
        default_permissions = ()

    def __str__(self):
        return _("ping [policy='%(config_name)s' user='%(user_name)s']") % {
            "config_name": self.config.name,
            "user_name": self.user.profile.main_character.character_name
            if self.user.profile.main_character
            else "None",
        }


class LeaveOfAbsence(models.Model):
    user = models.ForeignKey(User, on_delete=models.CASCADE)
    approver = models.ForeignKey(
        User, on_delete=models.SET_NULL, blank=True, null=True, related_name="+"
    )
    start = models.DateField(help_text=_("The start of the leave of absence."))
    end = models.DateField(
        blank=True,
        null=True,
        help_text=_(
            "The end of the leave of absence. Leave blank for an indefinite leave."
        ),
    )
    notes = models.TextField(blank=True)

    class Meta:
        default_permissions = ()

    def save(self, *args, **kwargs):
        orig = LeaveOfAbsence.objects.filter(pk=self.pk).first()
        # find possible configs that could apply to the user, then alert based on that
        configs = []
        for config in InactivityPingConfig.objects.all():
            if config.is_applicable_to(self.user):
                configs.append(config)
        webhooks = Webhook.objects.filter(
            Q(ping_configs__in=configs) | Q(ping_configs=None), Q(is_active=True)
        )
        if self._state.adding:

            for webhook in webhooks:
                if str(Webhook.NOTIFICATION_TYPE_LOA_NEW) in webhook.notification_types:
                    if webhook.webhook_type == Webhook.WEBHOOK_TYPE_DISCORD:
                        hook = dhooks_lite.Webhook(webhook.url)
                        hook.execute(
                            "**%(user_name)s** has submitted a new leave of absence from **%(start)s** to **%(end)s**"
                            % {
                                "user_name": self.user.profile.main_character.character_name,
                                "start": self.start,
                                "end": self.end if self.end else "—",
                            }
                        )
        elif self.approver and not orig.approver:
            for webhook in webhooks:
                if (
                    str(Webhook.NOTIFICATION_TYPE_LOA_APPROVED)
                    in webhook.notification_types
                ):
                    if webhook.webhook_type == Webhook.WEBHOOK_TYPE_DISCORD:
                        hook = dhooks_lite.Webhook(webhook.url)
                        hook.execute(
                            "**%(user_name)s**'s leave of absence from **%(start)s** to **%(end)s** has been approved by **%(approver_name)s**"
                            % {
                                "user_name": self.user.profile.main_character.character_name,
                                "approver_name": self.approver.profile.main_character.character_name,
                                "start": self.start,
                                "end": self.end if self.end else "—",
                            }
                        )

        super(LeaveOfAbsence, self).save(*args, **kwargs)

    def __str__(self):
        return _("%(user_name)s's leave starting %(start)s") % {
            "start": self.start,
            "user_name": self.user.profile.main_character.character_name,
        }

    def clean(self):
        if self.end and self.end < self.start:
            raise ValidationError(_("End date must be after start date."))


class Webhook(models.Model):
    NOTIFICATION_TYPE_INACTIVE_USER = 1
    NOTIFICATION_TYPE_LOA_NEW = 10
    NOTIFICATION_TYPE_LOA_APPROVED = 11

    NOTIFICATION_TYPE_CHOICES = [
        (NOTIFICATION_TYPE_INACTIVE_USER, "Inactive User"),
        (NOTIFICATION_TYPE_LOA_NEW, "Leave of Absence - Created"),
        (NOTIFICATION_TYPE_LOA_APPROVED, "Leave of Absence - Approved"),
    ]

    WEBHOOK_TYPE_DISCORD = 1

    WEBHOOK_TYPE_CHOICES = [
        (WEBHOOK_TYPE_DISCORD, _("Discord Webhook")),
    ]

    name = models.CharField(
        max_length=64, unique=True, help_text="short name to identify this webhook"
    )

    notification_types = MultiSelectField(
        choices=NOTIFICATION_TYPE_CHOICES,
        help_text=("only notifications of the selected types are sent to this webhook"),
    )

    ping_configs = models.ManyToManyField(
        InactivityPingConfig,
        blank=True,
        help_text="The inactivity policies to alert for. If left blank, all policies are alerted for.",
    )

    url = models.CharField(
        max_length=255,
        unique=True,
        help_text=(
            "URL of this webhook, e.g. "
            "https://discordapp.com/api/webhooks/123456/abcdef"
        ),
    )
    webhook_type = models.IntegerField(
        choices=WEBHOOK_TYPE_CHOICES,
        default=WEBHOOK_TYPE_DISCORD,
        help_text="type of this webhook",
    )
    is_active = models.BooleanField(
        default=True,
        help_text="whether notifications are currently sent to this webhook",
    )



import datetime as dt
from unittest.mock import patch

from app_utils.testdata_factories import UserMainFactory
from app_utils.testing import NoSocketsTestCase

from django.utils.timezone import now

from memberaudit.tests.testdata.factories import (
    create_character_from_user,
    create_online_status,
)

from ..tasks import check_inactivity_for_user
from .factories import (
    InactivityPingConfigFactory,
    InactivityPingFactory,
    LeaveOfAbsenceFactory,
)

TASKS_PATH = "inactivity.tasks"


@patch(TASKS_PATH + ".send_inactivity_ping")
class TestCheckInactivityForUser(NoSocketsTestCase):
    def test_should_ping_for_inactive_user(self, mock_send_inactivity_ping):
        # given
        user = UserMainFactory()
        character = create_character_from_user(user=user)
        last_login = now() - dt.timedelta(days=4)
        create_online_status(
            character=character,
            last_login=last_login,
            last_logout=last_login + dt.timedelta(hours=4),
        )
        InactivityPingConfigFactory(days=3)
        # when
        check_inactivity_for_user(user_pk=user.pk)
        # then
        self.assertTrue(mock_send_inactivity_ping.called)

    def test_should_not_ping_for_active_user(self, mock_send_inactivity_ping):
        # given
        user = UserMainFactory()
        character = create_character_from_user(user=user)
        last_login = now() - dt.timedelta(days=1)
        create_online_status(
            character=character,
            last_login=last_login,
            last_logout=last_login + dt.timedelta(hours=4),
        )
        InactivityPingConfigFactory(days=3)
        # when
        check_inactivity_for_user(user_pk=user.pk)
        # then
        self.assertFalse(mock_send_inactivity_ping.called)

    def test_should_not_ping_for_user_without_character(
        self, mock_send_inactivity_ping
    ):
        # given
        user = UserMainFactory()
        InactivityPingConfigFactory(days=3)
        # when
        check_inactivity_for_user(user_pk=user.pk)
        # then
        self.assertFalse(mock_send_inactivity_ping.called)

    def test_should_not_ping_for_excused_user(self, mock_send_inactivity_ping):
        # given
        user = UserMainFactory()
        character = create_character_from_user(user=user)
        last_login = now() - dt.timedelta(days=4)
        create_online_status(
            character=character,
            last_login=last_login,
            last_logout=last_login + dt.timedelta(hours=4),
        )
        LeaveOfAbsenceFactory(
            user=user, start=last_login, end=now() + dt.timedelta(days=3)
        )
        InactivityPingConfigFactory(days=3)
        # when
        check_inactivity_for_user(user_pk=user.pk)
        # then
        self.assertFalse(mock_send_inactivity_ping.called)

    def test_should_not_ping_when_already_pinged(self, mock_send_inactivity_ping):
        # given
        user = UserMainFactory()
        character = create_character_from_user(user=user)
        last_login = now() - dt.timedelta(days=4)
        create_online_status(
            character=character,
            last_login=last_login,
            last_logout=last_login + dt.timedelta(hours=4),
        )
        config = InactivityPingConfigFactory(days=3)
        InactivityPingFactory(user=user, config=config)
        # when
        check_inactivity_for_user(user_pk=user.pk)
        # then
        self.assertFalse(mock_send_inactivity_ping.called)



import datetime

from django.contrib import messages
from django.contrib.auth.decorators import login_required, permission_required
from django.db.models import Q
from django.http import HttpResponseNotFound, JsonResponse
from django.shortcuts import redirect, render
from django.utils.html import format_html
from django.utils.translation import gettext_lazy as _
from django.views.decorators.http import require_POST

from .forms import CreateRequestForm
from .models import LeaveOfAbsence


@login_required
@permission_required("inactivity.basic_access")
def index(request):

    context = {}
    return render(request, "inactivity/index.html", context)


@login_required
@permission_required("inactivity.manage_leave")
def manage(request):

    context = {}
    return render(request, "inactivity/manage.html", context)


def convert_loa(loa: LeaveOfAbsence) -> dict:
    return {
        "user": loa.user.profile.main_character.character_name,
        "start": loa.start,
        "end": loa.end if loa.end else "&mdash;",
        "approved": loa.approver is not None,
        "pk": loa.pk,
        "notes": loa.notes,
    }


@login_required
@permission_required("inactivity.basic_access")
def view_loa_request(request, request_id):
    if request.user.has_perm("inactivity.manage_leave"):
        candidate = LeaveOfAbsence.objects.filter(pk=request_id).first()
    else:
        candidate = LeaveOfAbsence.objects.filter(
            user=request.user, pk=request_id
        ).first()
    if candidate:
        context = {"request": convert_loa(candidate)}

        return render(request, "inactivity/modals/view_request_content.html", context)

    else:
        return HttpResponseNotFound("<h1>Request not found</h1>")


@login_required
@permission_required("inactivity.basic_access")
def list_loa_requests(request):
    results = []
    now = datetime.datetime.now(datetime.timezone.utc)

    for req in LeaveOfAbsence.objects.filter(
        Q(user=request.user), Q(end=None) | Q(end__gt=now)
    ):
        results.append(convert_loa(req))

    return JsonResponse(results, safe=False)


@login_required
@permission_required("inactivity.manage_leave")
def list_pending_loa_requests(request):
    results = []
    now = datetime.datetime.now(datetime.timezone.utc)

    for req in LeaveOfAbsence.objects.filter(
        Q(end=None) | Q(end__gt=now), Q(approver=None)
    ):
        results.append(convert_loa(req))

    return JsonResponse(results, safe=False)


@login_required
@permission_required("inactivity.manage_leave")
def approve_loa_request(request, request_id):
    req = LeaveOfAbsence.objects.get(pk=request_id)
    req.approver = request.user
    req.save()
    messages.success(
        request,
        format_html(
            _("Your have appproved %(user)s's leave request.")
            % {
                "user": format_html(
                    "<strong>{}</strong>",
                    req.user.profile.main_character.character_name,
                ),
            }
        ),
    )
    return redirect("inactivity:manage_requests")


@login_required
@permission_required("inactivity.basic_access")
@require_POST
def cancel_loa_request(request, request_id):
    if request.user.has_perm("inactivity.manage_leave"):
        candidate = LeaveOfAbsence.objects.filter(pk=request_id).first()
    else:
        candidate = LeaveOfAbsence.objects.filter(
            user=request.user, pk=request_id
        ).first()
    if candidate:
        candidate.delete()
        messages.info(
            request,
            format_html(
                _(
                    "Your leave of absence request from %(start)s to %(end)s has been deleted."
                )
                % {
                    "start": format_html("<strong>{}</strong>", candidate.start),
                    "end": format_html("<strong>{}</strong>", candidate.end),
                }
            ),
        )
        return redirect("inactivity:index")
    else:
        messages.error(
            request,
            format_html(_("No leave of absence request matched your request.")),
        )
        return redirect("inactivity:index")


@login_required
@permission_required("inactivity.basic_access")
def create_loa_request(request):
    if request.method == "GET":
        create_form = CreateRequestForm()
        context = {"create_form": create_form}
        return render(request, "inactivity/create_loa.html", context)
    elif request.method == "POST":
        create_form = CreateRequestForm(request.POST)
        model = create_form.save(commit=False)
        model.user = request.user
        model.save()
        messages.info(
            request,
            format_html(
                _(
                    "Your leave of absence request from %(start)s to %(end)s has been submitted for review."
                )
                % {
                    "start": format_html("<strong>{}</strong>", model.start),
                    "end": format_html("<strong>{}</strong>", model.end),
                }
            ),
        )
        return redirect("inactivity:index")



# Generated by Django 3.1.2 on 2021-01-22 00:24

import multiselectfield.db.fields

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("auth", "0012_alter_user_first_name_max_length"),
        ("authentication", "0017_remove_fleetup_permission"),
        ("inactivity", "0002_add_manage_leave_perm"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="inactivitypingconfig",
            options={
                "default_permissions": (),
                "verbose_name": "inactivity policy",
                "verbose_name_plural": "inactivity policies",
            },
        ),
        migrations.AlterField(
            model_name="inactivitypingconfig",
            name="groups",
            field=models.ManyToManyField(
                blank=True,
                help_text="Groups subject to the inactivity policy. If empty, applies to all groups.",
                related_name="_inactivitypingconfig_groups_+",
                to="auth.Group",
            ),
        ),
        migrations.AlterField(
            model_name="inactivitypingconfig",
            name="name",
            field=models.CharField(
                help_text="Internal name for the inactivity policy. Must be unique.",
                max_length=48,
                unique=True,
            ),
        ),
        migrations.AlterField(
            model_name="inactivitypingconfig",
            name="states",
            field=models.ManyToManyField(
                blank=True,
                help_text="States subject to the inactivity policy. If empty, applies to all states.",
                related_name="_inactivitypingconfig_states_+",
                to="authentication.State",
            ),
        ),
        migrations.CreateModel(
            name="Webhook",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "name",
                    models.CharField(
                        help_text="short name to identify this webhook",
                        max_length=64,
                        unique=True,
                    ),
                ),
                (
                    "notification_types",
                    multiselectfield.db.fields.MultiSelectField(
                        choices=[
                            (1, "Inactive User"),
                            (10, "Leave of Absence - Created"),
                            (11, "Leave of Absence - Approved"),
                        ],
                        help_text="only notifications of the selected types are sent to this webhook",
                        max_length=7,
                    ),
                ),
                (
                    "url",
                    models.CharField(
                        help_text="URL of this webhook, e.g. https://discordapp.com/api/webhooks/123456/abcdef",
                        max_length=255,
                        unique=True,
                    ),
                ),
                (
                    "webhook_type",
                    models.IntegerField(
                        choices=[(1, "Discord Webhook")],
                        default=1,
                        help_text="type of this webhook",
                    ),
                ),
                (
                    "is_active",
                    models.BooleanField(
                        default=True,
                        help_text="whether notifications are currently sent to this webhook",
                    ),
                ),
                (
                    "ping_configs",
                    models.ManyToManyField(
                        blank=True,
                        help_text="The inactivity policies to alert for. If left blank, all policies are alerted for.",
                        to="inactivity.InactivityPingConfig",
                    ),
                ),
            ],
        ),
    ]



from django.conf import settings
from django.contrib.auth.models import User

from allianceauth.notifications import notify


def notify_user(user_pk: int, message: str):
    if "aadiscordbot" in settings.INSTALLED_APPS:
        import aadiscordbot.tasks

        aadiscordbot.tasks.send_direct_message_by_user_id.delay(user_pk, message)
    else:
        notify(
            User.objects.get(pk=user_pk), "Inactivity Notification", message, "danger"
        )
